@@ -9,7 +9,6 @@
 
 #include "loggingcategory.h"
 #include <QAbstractItemModel>
-#include <QDebug>
 #include <QGuiApplication>
 #include <QPropertyAnimation>
 #include <QQmlComponent>
@@ -19,8 +18,6 @@
 
 #include "units.h"
 
-QHash<QObject *, ColumnViewAttached *> ColumnView::m_attachedObjects = QHash<QObject *, ColumnViewAttached *>();
-
 class QmlComponentsPoolSingleton
 {
 public:
@@ -46,11 +43,15 @@ QmlComponentsPool *QmlComponentsPoolSingleton::instance(QQmlEngine *engine)
 
     componentPool = new QmlComponentsPool(engine);
 
-    QObject::connect(componentPool, &QObject::destroyed, engine, [engine]() {
+    const auto removePool = [engine]() {
+        // NB: do not derefence engine. it may be dangling already!
         if (privateQmlComponentsPoolSelf) {
             privateQmlComponentsPoolSelf->m_instances.remove(engine);
         }
-    });
+    };
+    QObject::connect(engine, &QObject::destroyed, engine, removePool);
+    QObject::connect(componentPool, &QObject::destroyed, componentPool, removePool);
+
     privateQmlComponentsPoolSelf->m_instances[engine] = componentPool;
     return componentPool;
 }
@@ -91,6 +92,7 @@ QtObject {
     m_instance = component->create();
     // qCWarning(KirigamiLog)<<component->errors();
     Q_ASSERT(m_instance);
+    m_instance->setParent(this);
 
     m_separatorComponent = m_instance->property("separator").value<QQmlComponent *>();
     Q_ASSERT(m_separatorComponent);
@@ -402,7 +404,7 @@ qreal ContentItem::childWidth(QQuickItem *child)
         return qRound(parentItem()->width());
 
     } else if (attached->fillWidth()) {
-        return qRound(qBound(m_columnWidth, (parentItem()->width() - attached->reservedSpace()), parentItem()->width()));
+        return qRound(qBound(m_columnWidth, (parentItem()->width() - attached->reservedSpace()), std::max(m_columnWidth, parentItem()->width())));
 
     } else if (m_columnResizeMode == ColumnView::FixedColumns) {
         return qRound(qMin(parentItem()->width(), m_columnWidth));
@@ -571,10 +573,10 @@ void ContentItem::updateVisibleItems()
     if (newItems != m_visibleItems) {
         m_visibleItems = newItems;
         Q_EMIT m_view->visibleItemsChanged();
-        if (!newItems.isEmpty() && m_visibleItems.first() != oldFirstVisibleItem) {
+        if (!m_visibleItems.isEmpty() && m_visibleItems.first() != oldFirstVisibleItem) {
             Q_EMIT m_view->firstVisibleItemChanged();
         }
-        if (!newItems.isEmpty() && m_visibleItems.last() != oldLastVisibleItem) {
+        if (!m_visibleItems.isEmpty() && m_visibleItems.last() != oldLastVisibleItem) {
             Q_EMIT m_view->lastVisibleItemChanged();
         }
     }
@@ -611,7 +613,7 @@ void ContentItem::forgetItem(QQuickItem *item)
     m_view->polish();
 
     if (index <= m_view->currentIndex()) {
-        m_view->setCurrentIndex(qBound(0, index - 1, m_items.count() - 1));
+        m_view->setCurrentIndex(m_items.isEmpty() ? 0 : qBound(0, index - 1, m_items.count() - 1));
     }
     Q_EMIT m_view->countChanged();
 }
@@ -624,6 +626,7 @@ QQuickItem *ContentItem::ensureSeparator(QQuickItem *item)
         separatorItem = qobject_cast<QQuickItem *>(
             privateQmlComponentsPoolSelf->instance(qmlEngine(item))->m_separatorComponent->beginCreate(QQmlEngine::contextForObject(item)));
         if (separatorItem) {
+            separatorItem->setParent(this);
             separatorItem->setParentItem(item);
             separatorItem->setZ(9999);
             separatorItem->setProperty("column", QVariant::fromValue(item));
@@ -643,6 +646,7 @@ QQuickItem *ContentItem::ensureRightSeparator(QQuickItem *item)
         separatorItem = qobject_cast<QQuickItem *>(
             QmlComponentsPoolSingleton::instance(qmlEngine(item))->m_rightSeparatorComponent->beginCreate(QQmlEngine::contextForObject(item)));
         if (separatorItem) {
+            separatorItem->setParent(this);
             separatorItem->setParentItem(item);
             separatorItem->setZ(9999);
             separatorItem->setProperty("column", QVariant::fromValue(item));
@@ -703,11 +707,19 @@ void ContentItem::itemChange(QQuickItem::ItemChange change, const QQuickItem::It
     QQuickItem::itemChange(change, value);
 }
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 void ContentItem::geometryChanged(const QRectF &newGeometry, const QRectF &oldGeometry)
 {
     updateVisibleItems();
     QQuickItem::geometryChanged(newGeometry, oldGeometry);
 }
+#else
+void ContentItem::geometryChange(const QRectF &newGeometry, const QRectF &oldGeometry)
+{
+    updateVisibleItems();
+    QQuickItem::geometryChange(newGeometry, oldGeometry);
+}
+#endif
 
 void ContentItem::syncItemsOrder()
 {
@@ -1230,6 +1242,7 @@ QQuickItem *ColumnView::removeItem(QQuickItem *item)
         item->setParentItem(attached ? attached->originalParent() : nullptr);
     }
 
+    Q_EMIT contentChildrenChanged();
     Q_EMIT itemRemoved(item);
 
     return item;
@@ -1283,7 +1296,11 @@ ColumnViewAttached *ColumnView::qmlAttachedProperties(QObject *object)
     return new ColumnViewAttached(object);
 }
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 void ColumnView::geometryChanged(const QRectF &newGeometry, const QRectF &oldGeometry)
+#else
+void ColumnView::geometryChange(const QRectF &newGeometry, const QRectF &oldGeometry)
+#endif
 {
     m_contentItem->setY(m_topPadding);
     m_contentItem->setHeight(newGeometry.height() - m_topPadding - m_bottomPadding);
@@ -1291,7 +1308,11 @@ void ColumnView::geometryChanged(const QRectF &newGeometry, const QRectF &oldGeo
     polish();
 
     m_contentItem->updateVisibleItems();
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     QQuickItem::geometryChanged(newGeometry, oldGeometry);
+#else
+    QQuickItem::geometryChange(newGeometry, oldGeometry);
+#endif
 }
 
 bool ColumnView::childMouseEventFilter(QQuickItem *item, QEvent *event)
@@ -1406,12 +1427,10 @@ bool ColumnView::childMouseEventFilter(QQuickItem *item, QEvent *event)
         me->setAccepted(m_dragging);
 
         return m_dragging && !verticalScrollIntercepted;
-        break;
     }
     case QEvent::MouseButtonRelease: {
         QMouseEvent *me = static_cast<QMouseEvent *>(event);
-
-        if (!m_acceptsMouse && me->source() == Qt::MouseEventNotSynthesized) {
+        if (item->property("preventStealing").toBool()) {
             return false;
         }
 
@@ -1425,23 +1444,23 @@ bool ColumnView::childMouseEventFilter(QQuickItem *item, QEvent *event)
             return true;
         }
 
+        if (!m_acceptsMouse && me->source() == Qt::MouseEventNotSynthesized) {
+            return false;
+        }
+
         if (me->button() != Qt::LeftButton) {
             return false;
         }
 
         m_mouseDown = false;
 
-        m_contentItem->snapToItem();
-        m_contentItem->m_lastDragDelta = 0;
         if (m_dragging) {
+            m_contentItem->snapToItem();
+            m_contentItem->m_lastDragDelta = 0;
             m_dragging = false;
             Q_EMIT draggingChanged();
         }
 
-        if (item->property("preventStealing").toBool()) {
-            return false;
-        }
-
         event->accept();
 
         // if a drag happened, don't pass the event
@@ -1450,7 +1469,6 @@ bool ColumnView::childMouseEventFilter(QQuickItem *item, QEvent *event)
 
         me->setAccepted(block);
         return block;
-        break;
     }
     default:
         break;
@@ -1564,7 +1582,7 @@ void ColumnView::mouseUngrabEvent()
 void ColumnView::classBegin()
 {
     auto syncColumnWidth = [this]() {
-        m_contentItem->m_columnWidth = privateQmlComponentsPoolSelf->instance(qmlEngine(this))->m_units->property("gridUnit").toInt() * 20;
+        m_contentItem->m_columnWidth = privateQmlComponentsPoolSelf->instance(qmlEngine(this))->m_units->gridUnit() * 20;
         Q_EMIT columnWidthChanged();
     };
 
@@ -1572,7 +1590,7 @@ void ColumnView::classBegin()
     syncColumnWidth();
 
     auto syncDuration = [this]() {
-        m_contentItem->m_slideAnim->setDuration(QmlComponentsPoolSingleton::instance(qmlEngine(this))->m_units->property("longDuration").toInt());
+        m_contentItem->m_slideAnim->setDuration(QmlComponentsPoolSingleton::instance(qmlEngine(this))->m_units->longDuration());
         Q_EMIT scrollDurationChanged();
     };
 
@@ -1627,7 +1645,11 @@ void ColumnView::contentChildren_append(QQmlListProperty<QQuickItem> *prop, QQui
     item->setParentItem(view->m_contentItem);
 }
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 int ColumnView::contentChildren_count(QQmlListProperty<QQuickItem> *prop)
+#else
+qsizetype ColumnView::contentChildren_count(QQmlListProperty<QQuickItem> *prop)
+#endif
 {
     ColumnView *view = static_cast<ColumnView *>(prop->object);
     if (!view) {
@@ -1637,7 +1659,11 @@ int ColumnView::contentChildren_count(QQmlListProperty<QQuickItem> *prop)
     return view->m_contentItem->m_items.count();
 }
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 QQuickItem *ColumnView::contentChildren_at(QQmlListProperty<QQuickItem> *prop, int index)
+#else
+QQuickItem *ColumnView::contentChildren_at(QQmlListProperty<QQuickItem> *prop, qsizetype index)
+#endif
 {
     ColumnView *view = static_cast<ColumnView *>(prop->object);
     if (!view) {
@@ -1702,7 +1728,11 @@ void ColumnView::contentData_append(QQmlListProperty<QObject> *prop, QObject *ob
     }
 }
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 int ColumnView::contentData_count(QQmlListProperty<QObject> *prop)
+#else
+qsizetype ColumnView::contentData_count(QQmlListProperty<QObject> *prop)
+#endif
 {
     ColumnView *view = static_cast<ColumnView *>(prop->object);
     if (!view) {
@@ -1712,7 +1742,11 @@ int ColumnView::contentData_count(QQmlListProperty<QObject> *prop)
     return view->m_contentData.count();
 }
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 QObject *ColumnView::contentData_at(QQmlListProperty<QObject> *prop, int index)
+#else
+QObject *ColumnView::contentData_at(QQmlListProperty<QObject> *prop, qsizetype index)
+#endif
 {
     ColumnView *view = static_cast<ColumnView *>(prop->object);
     if (!view) {
